searchNodes=[{"doc":"An HTTP caching library http_cache is a stateless Erlang HTTP caching library that implements the various HTTP RFCs related to caching. Modules http_cache exposes functions to cache backend responses, get cached responses whenever they can be served, and invalidate previously stored responses. http_cache_store is the behaviour to be implemented by stores. http_cache_store_process is an example store that stores cached responses in the current process and is mainly used for testing purpose. Telemetry events All time measurements are in microseconds. The following events are emitted by http_cache : [http_cache, lookup] when http_cache:get/2 is called. Measurements: total_time : the total time of the lookup store_lookup_time : time taken to query the store for suitable responses response_selection_time : time to select the best response among suitable responses. A high value can indicate the presence of too many variants candidate_count : the number of candidate responses that are returned by the store. A high value can indicate the presence of too many variants decompress_time : time spend decompressing the response range_time : time spend constructing a range response Metadata: freshness : one of fresh , stale , must_revalidate or miss [http_cache, cache] when http_cache:cache/3 or http_cache:cache/4 is called. Measurements: total_time : the total time of the caching operation store_save_time : time taken to save the response into the store compress_time : time spend compressing the response. This happens when the auto_compress option is used decompress_time : time spend decompressing the response. This happens when the auto_compress option is used but the client does not support compression and the result, stored compressed, has to be returned uncompressed range_time : time spend constructing a range response Metadata: cacheable : true if the response was cacheable (and cached), false otherwise [http_cache, invalidation] when http_cache:invalidate_url/2 or http_cache:invalidate_by_alternate_key/2 is called. Measurements: duration : the time it took to invalidate entries count : the number of entries invalidated if the store supports returning this value Metadata: type : invalidate_by_url or invalidate_by_alternate_key [http_cache, store, error] informs about errors of the store. Measurements: none Metadata: type : cache , invalidate_by_url` or `invalidate_by_alternate_key reason : an erlang term that gives the error reason [http_cache, compress_operation] whenever a compress operation is performed on an HTTP response. Measurements: none Metadata: alg : gzip (which is the only supported algorithm at the moment) [http_cache, decompress_operation] whenever a decompress operation is performed on an HTTP response. Measurements: none Metadata: alg : gzip (which is the only supported algorithm at the moment)","ref":"http_cache.html","title":"http_cache","type":"module"},{"doc":"Caches a response This function never returns an error, even when the backend store returns one. Instead it returns {ok, response()} when the response is cacheable (even if an error occurs to actually save it) or not_cacheable when the response cannot be cached. When {ok, response()} is returned, the response should be returned to the client instead of the initial response that was passed as a parameter, because it is transformed accordingly to the options passed: it can be compressed or uncompressed, and it will be returned as a range response if the request is a range request and the backend doesn't support it and returned a full response. This function shall be called with any response, even those known to be not cacheable, such as DELETE requests, because such non-cacheable request can still have side effects on other cached objects (see https://www.rfc-editor.org/rfc/rfc9111.html#name-invalidating-stored-respons ). In this example, a successful DELETE request triggers the invalidation of cached results of the deleted object with the same URL.","ref":"http_cache.html#cache/3","title":"http_cache.cache/3","type":"function"},{"doc":"Caches a response when revalidating Similar to cache/3 , but to be used when revalidating a response, when get/2 return a :must_revalidate response. The Response parameter is the response received from the origin server, and the RevalidatedResponse parameter is the previously :must_revalidate response that is being revalidated. When the returned response is a 304 (not modified) response, stored responses are updated and a response is returned from the 2 responses passed as a parameter. It's recommended to use the response returned by this function, because the 304 response is used to update headers of the revalidated response. Otherwise, cache/3 is called.","ref":"http_cache.html#cache/4","title":"http_cache.cache/4","type":"function"},{"doc":"Gets a response from the cache for the given answer The function returns one of: {fresh, {http_cache_store_behaviour:response_ref(), response()}} : the response is fresh and can be returned directly to the client. {stale, {http_cache_store_behaviour:response_ref(), response()}} : the response is stale but can be directly returned to the client. Stale responses that are cached but cannot be returned do to unfulfilled condition are not returned. By default, a stale response is returned only when there's a max-stale header in the request. See the following option to enable returning stale response in other cases: allow_stale_while_revalidate allow_stale_if_error origin_unreachable {must_revalidate, {http_cache_store_behaviour:response_ref(), response()}} : the response must be revalidated. miss : no suitable response was found. Using this function does not automatically notify the response was returned. Therefore, use notify_response_used/2 with the returned response reference when a cached response is used.","ref":"http_cache.html#get/2","title":"http_cache.get/2","type":"function"},{"doc":"Invalidates all responses stored with the alternate key","ref":"http_cache.html#invalidate_by_alternate_key/2","title":"http_cache.invalidate_by_alternate_key/2","type":"function"},{"doc":"Invalidates all responses for a URL This includes all variants and all responses for all HTTP methods.","ref":"http_cache.html#invalidate_url/2","title":"http_cache.invalidate_url/2","type":"function"},{"doc":"Notifies a response is currently being downloaded For future use, does not do anything at the moment.","ref":"http_cache.html#notify_downloading/3","title":"http_cache.notify_downloading/3","type":"function"},{"doc":"Notifies the backend that a response was used Some backends, such as LRU backends, need to update metadata (in that case: last used time) when a response is used.","ref":"http_cache.html#notify_response_used/2","title":"http_cache.notify_response_used/2","type":"function"},{"doc":"Alternate key attached to a stored response Used to invalidate by alternate key (e.g. to invalidate all the images if the image alternate key if set to all images).","ref":"http_cache.html#t:alternate_key/0","title":"http_cache.alternate_key/0","type":"type"},{"doc":"Request or response body","ref":"http_cache.html#t:body/0","title":"http_cache.body/0","type":"type"},{"doc":"Request or response headers A header can appear more than once, this is allowed by HTTP","ref":"http_cache.html#t:headers/0","title":"http_cache.headers/0","type":"type"},{"doc":"","ref":"http_cache.html#t:invalidation_result/0","title":"http_cache.invalidation_result/0","type":"type"},{"doc":"An HTTP method, for example &quot;PATCH&quot;","ref":"http_cache.html#t:method/0","title":"http_cache.method/0","type":"type"},{"doc":"Options passed to the functions of this module alternate_keys : alternate keys associated with the stored request. Requests can then be invalidated by alternate key with invalidate_by_alternate_key/2 . Use by cache/3 and cache/4 . allow_stale_while_revalidate : allows returning valid stale response while revalidating. Used by get/2 . Defaults to false . allow_stale_if_error : allows returning valid stale response when an error occurs. See https://datatracker.ietf.org/doc/html/rfc5861#section-3 . Used by get/2 . Defaults to false . auto_accept_encoding : automatically selects an acceptable response based on accept-encoding and content-encoding headers. Compressed response vary on the exact value accept-encoding header. For example, gzip, brotli , brotli, gzip , brotli,gzip and gzip;q=1.0, brotli;q=1.0 are equivalent but considered different because their string representation do not match. So, if a response of a request with the accept-encoding: gzip is cached, none of the abovementionned variations would result in returning the cached response. When set to true , this options allows automatically returning acceptable content when available even when headers don't exactly match. Doesn't take priority into account (except for priority 0 which is discarded). Used by get/2 . Defaults to false . auto_compress : automatically compresses decompressed text responses with gzip. This can help with reducing the size of stored content. Moreover, most browsers do support gzip encoding. When this option is used, auto_decompress is automatically set to true as well. Does not compress responses with strong etags (see https://bz.apache.org/bugzilla/show_bug.cgi?id=63932 ). Used by cache/3 and cache/4 . Defaults to false . auto_compress_mime_types : the list of mime-types that are compressed when auto_compress is used. Used by cache/3 and cache/4 . Defaults to [&lt;&lt;&quot;text/html&quot;&gt;&gt;, &lt;&lt;&quot;text/css&quot;&gt;&gt;, &lt;&lt;&quot;text/plain&quot;&gt;&gt;, &lt;&lt;&quot;text/xml&quot;&gt;&gt;, &lt;&lt;&quot;text/javascript&quot;&gt;&gt;, &lt;&lt;&quot;application/javascript&quot;&gt;&gt;, &lt;&lt;&quot;application/json&quot;&gt;&gt;, &lt;&lt;&quot;application/ld+json&quot;&gt;&gt;, &lt;&lt;&quot;application/xml&quot;&gt;&gt;, &lt;&lt;&quot;application/xhtml+xml&quot;&gt;&gt;, &lt;&lt;&quot;application/rss+xml&quot;&gt;&gt;, &lt;&lt;&quot;application/atom+xml&quot;&gt;&gt;, &lt;&lt;&quot;image/svg+xml&quot;&gt;&gt;, &lt;&lt;&quot;font/ttf&quot;&gt;&gt;, &lt;&lt;&quot;font/eot&quot;&gt;&gt;, &lt;&lt;&quot;font/otf&quot;&gt;&gt;, &lt;&lt;&quot;font/opentype&quot;&gt;&gt; ] auto_decompress : automatically decompresses stored gzip responses when the client does not support compression. Does not decompress responses with strong etags (see https://bz.apache.org/bugzilla/show_bug.cgi?id=63932 ). Used by get/2 , cache/3 and cache/4 . Defaults to false . bucket : an Erlang term to differentiate between different caches. For instance, when what needs to use several private caches, this option can be used to differentiate the cached responses and prevent them from being mixed up, potentially leaking private data. Used by get/2 , cache/3 and cache/4 . Defaults to the atom default . compression_threshold : compression threshold in bytes. Compressing a very tiny response can result in actually bigger response (in addition to the performance hit of compression it). Although there's no additional cost when this library serves a compressed file, but it has a cost on the client that has to decompress it. This is why the default value is so high: we want to make sure that it's worth performing compression and decompression. See further discussion: https://webmasters.stackexchange.com/questions/31750/what-is-recommended-minimum-object-size-for-gzip-performance-benefits . Used by cache/3 and cache/4 . Defaults to 1000 . origin_unreachable : indicates that the current cache using this library is unable to reach the origin server. In this case, a stale response can be returned even if the HTTP cache headers do not explicitely allow it. Used by get/2 . Defaults to false . default_ttl : the default TTL, in seconds. This value is used when no TTL information is found in the response, but the response is cacheable by default (see https://datatracker.ietf.org/doc/html/rfc7231#section-6.1 ). Used by cache/3 and cache/4 . Defaults to 120 . default_grace : the amount of time an expired response is kept in the cache. Such a response is called a stale response, and can be returned in some circumstances, for instance when the origin server returns an 5xx error and stale-if-error header is used. Use by cache/3 and cache/4 . Defaults to 120 . ignore_query_params_order : when a response is cached, a request key is computed based on the method, URL and body. This option allows to keep the same request key for URLs whose parameters are identical, but in different order. This helps increasing cache hit if URL parameter order doesn't matter. Used by get/2 , cache/3 and cache/4 . Defaults to false . max_ranges : maximum number of range sets accepted when responding to a range request. This is limited to avoid DOS attack by a client. See https://datatracker.ietf.org/doc/html/rfc7233#section-6.1 . Used by get/2 . Defaults to 100 . prevent_set_cookie : when set to auto , raises when the set-cookie is used on shared caches. When set to true , always raises in this case. When set to false , never raises even for shared caches. Used by cache/3 and cache/4 . Defaults to auto . store : required , the store backend's module name. Used by all functions, no defaults. store_opts : the store backend's options. Used by all functions, defaults to [] . type : cache type. shared or private . A CDN is an example of a shared cache. A browser cache is an example of a private cache. Used by get/2 , cache/3 and cache/4 . Defaults to shared . request_time : the time the request was initiated, as a UNIX timestamp in seconds. Setting this timestamp helps correcting the age of the request between the time the request was made and the time the response was received and cached, which can be several seconds. Used by cache/3 and cache/4 .","ref":"http_cache.html#t:opts/0","title":"http_cache.opts/0","type":"type"},{"doc":"An HTTP request","ref":"http_cache.html#t:request/0","title":"http_cache.request/0","type":"type"},{"doc":"An HTTP response","ref":"http_cache.html#t:response/0","title":"http_cache.response/0","type":"type"},{"doc":"","ref":"http_cache.html#t:sendfile/0","title":"http_cache.sendfile/0","type":"type"},{"doc":"HTTP status","ref":"http_cache.html#t:status/0","title":"http_cache.status/0","type":"type"},{"doc":"UNIX timestamp in seconds","ref":"http_cache.html#t:timestamp/0","title":"http_cache.timestamp/0","type":"type"},{"doc":"","ref":"http_cache.html#t:type/0","title":"http_cache.type/0","type":"type"},{"doc":"An URL, with the schema, domain and optionally path","ref":"http_cache.html#t:url/0","title":"http_cache.url/0","type":"type"},{"doc":"An in-process implementation of http_cache_store This implementation is used in this library's test and can be used in your own tests. It stores data in the current process, and thus provides isolation. It cannot, of course, be used in real-life because data is discarded as soon as the process dies, and no cleanup ever happens. The save_in_file/0 function can be used to simulate a store that saves responses on the disk. It saves the responses to files in tmp and therefore cannot be used on non-UNIX systems.","ref":"http_cache_store_process.html","title":"http_cache_store_process","type":"module"},{"doc":"","ref":"http_cache_store_process.html#get_response/2","title":"http_cache_store_process.get_response/2","type":"function"},{"doc":"","ref":"http_cache_store_process.html#invalidate_by_alternate_key/2","title":"http_cache_store_process.invalidate_by_alternate_key/2","type":"function"},{"doc":"","ref":"http_cache_store_process.html#invalidate_url/2","title":"http_cache_store_process.invalidate_url/2","type":"function"},{"doc":"","ref":"http_cache_store_process.html#list_candidates/2","title":"http_cache_store_process.list_candidates/2","type":"function"},{"doc":"","ref":"http_cache_store_process.html#notify_response_used/2","title":"http_cache_store_process.notify_response_used/2","type":"function"},{"doc":"","ref":"http_cache_store_process.html#put/6","title":"http_cache_store_process.put/6","type":"function"},{"doc":"Saves response into a file When called before saving, it instructs this implementation to store the response in a file in /tmp . Not thaat it works only on UNIX systems.","ref":"http_cache_store_process.html#save_in_file/0","title":"http_cache_store_process.save_in_file/0","type":"function"},{"doc":"http_cache is a stateless Erlang HTTP caching library that implements the various HTTP RFCs related to caching. When caching, it analyses the response along with the associated request to determine whether the response is cacheable, and caches it if so. When looking up cached responses, it analyses the request and finds the most suitable response (that is, response that is the freshest and conforms to the vary header). It supports invalidating cached responses: by URL by alternate key It also supports: automatically compressing and decompressing with gzip. It allows saving space when storing response and reducing transmission time replying to conditional requests replying to range requests Finally, many telemetry events are emitted. See the documentation of the main module.","ref":"readme.html","title":"http_cache","type":"extras"},{"doc":"1 &gt; Req = { &lt;&lt; &quot;GET&quot; &gt;&gt; , &lt;&lt; &quot;http://example.org&quot; &gt;&gt; , [ ] , &lt;&lt; &gt;&gt; } . { &lt;&lt; &quot;GET&quot; &gt;&gt; , &lt;&lt; &quot;http://example.org&quot; &gt;&gt; , [ ] , &lt;&lt; &gt;&gt; } 2 &gt; Resp = { 200 , [ { &lt;&lt; &quot;content-type&quot; &gt;&gt; , &lt;&lt; &quot;text/plain&quot; &gt;&gt; } ] , &lt;&lt; &quot;Cache me&quot; &gt;&gt; } . { 200 , [ { &lt;&lt; &quot;content-type&quot; &gt;&gt; , &lt;&lt; &quot;text/plain&quot; &gt;&gt; } ] , &lt;&lt; &quot;Cache me&quot; &gt;&gt; } 3 &gt; Opts = \#{ store =&gt; http_cache_store_process , type =&gt; shared } . \#{ store =&gt; http_cache_store_process , type =&gt; shared } 4 &gt; http_cache : cache ( Req , Resp , Opts ) . { ok , { 200 , [ { &lt;&lt; &quot;content-type&quot; &gt;&gt; , &lt;&lt; &quot;text/plain&quot; &gt;&gt; } , { &lt;&lt; &quot;content-length&quot; &gt;&gt; , &lt;&lt; &quot;8&quot; &gt;&gt; } ] , &lt;&lt; &quot;Cache me&quot; &gt;&gt; } } 5 &gt; http_cache : get ( Req , Opts ) . { fresh , { { &lt;&lt; 21 , 255 , 141 , 93 , 218 , 86 , 217 , 58 , 55 , 246 , 85 , 151 , 223 , 133 , 134 , 248 , 212 , 121 , 102 , 151 , 176 , 244 , 210 , 11 , 46 , . . . &gt;&gt; , \#{ } } , { 200 , [ { &lt;&lt; &quot;content-type&quot; &gt;&gt; , &lt;&lt; &quot;text/plain&quot; &gt;&gt; } , { &lt;&lt; &quot;content-length&quot; &gt;&gt; , &lt;&lt; &quot;8&quot; &gt;&gt; } , { &lt;&lt; &quot;age&quot; &gt;&gt; , &lt;&lt; &quot;10&quot; &gt;&gt; } ] , &lt;&lt; &quot;Cache me&quot; &gt;&gt; } } } 6 &gt; http_cache : get ( Req , Opts ) . { must_revalidate , { { &lt;&lt; 21 , 255 , 141 , 93 , 218 , 86 , 217 , 58 , 55 , 246 , 85 , 151 , 223 , 133 , 134 , 248 , 212 , 121 , 102 , 151 , 176 , 244 , 210 , 11 , 46 , . . . &gt;&gt; , \#{ } } , { 200 , [ { &lt;&lt; &quot;content-type&quot; &gt;&gt; , &lt;&lt; &quot;text/plain&quot; &gt;&gt; } , { &lt;&lt; &quot;content-length&quot; &gt;&gt; , &lt;&lt; &quot;8&quot; &gt;&gt; } , { &lt;&lt; &quot;age&quot; &gt;&gt; , &lt;&lt; &quot;218&quot; &gt;&gt; } ] , &lt;&lt; &quot;Cache me&quot; &gt;&gt; } } } 7 &gt; http_cache : get ( Req , Opts ) . miss","ref":"readme.html#usage","title":"http_cache - Usage","type":"extras"},{"doc":"Responses have to be stored in a separate store backend (this library being stateless), such as: http_cache_store_memory : responses are stored in memory (ETS) http_cache_store_disk : responses are stored on disk. An application using the sendfile system call (such as plug_http_cache ) may benefit from the kernel's memory caching automatically Both are cluster-aware.","ref":"readme.html#store-backends","title":"http_cache - Store backends","type":"extras"},{"doc":"This library may store different responses for the same URL, following the directives of the &quot;vary&quot; header. For instance, if a response can be returned in English or in French, both versions can be cached as long as the &quot;vary&quot; header is correctly used. This can unfortunately result in an explosion of stored responses if the headers are not normalized. For instance, in this scenario where a site handles both these languages, a response will be stored for any of these requests that include an accept-language header: fr-CH, fr;q=0.9, en;q=0.8, de;q=0.7, *;q=0.5 fr-CH, fr;q=0.9, en;q=0.8, de;q=0.7,*;q=0.5 en de en, de en, de, fr en;q=1, de en;q=1, de;q=0.9 en;q=1, de;q=0.8 en;q=1, de;q=0.7 en;q=1, de;q=0.6 en;q=1, de;q=0.5 and so on, so potentially hundreds of stored responses for only 2 available responses (English or French versions). In this case, you probably want to apply normalization before caching, that is modify the accept-language header to have only the en or fr value set before using this library. See Best practices for using the Vary header for more guidance regarding this issue.","ref":"readme.html#header-normalisation","title":"http_cache - Header normalisation","type":"extras"},{"doc":"OTP24+ RFC5861 ( stale-if-error and stale-while-revalidate cache directives) is supported on latest development branch of cowlib (since this commit ) or in other words from version 2.12 (not released yet as April, 2022). Manually override dependency if you need to use it (you can take a look at this project's rebar.config file).","ref":"readme.html#support","title":"http_cache - Support","type":"extras"},{"doc":"RFC9111: HTTP Caching : [ ] 3. Storing Responses in Caches [x] 3.1. Storing Header and Trailer Fields [x] 3.2. Updating Stored Header Fields [ ] 3.3. Storing Incomplete Responses [ ] 3.4. Combining Partial Content [x] 3.5. Storing Responses to Authenticated Requests [x] 4. Constructing Responses from Caches [x] 4.1. Calculating Secondary Keys with Vary [x] 4.2. Freshness [x] 4.2.1. Calculating Freshness Lifetime [x] 4.2.2. Calculating Heuristic Freshness [x] 4.2.3. Calculating Age [x] 4.2.4. Serving Stale Responses [x] 4.3. Validation [x] 4.3.1. Sending a Validation Request [x] 4.3.2. Handling a Received Validation Request [x] 4.3.3. Handling a Validation Response [x] 4.3.4. Freshening Stored Responses upon Validation [x] 4.3.5. Freshening Responses via HEAD [x] 4.4. Invalidating Stored Responses [ ] 5. Header Field Definitions [x] 5.1. Age [x] 5.2. Cache-Control [x] 5.2.1. Request Cache-Control Directives [x] 5.2.1.1. max-age [x] 5.2.1.2. max-stale [x] 5.2.1.3. min-fresh [x] 5.2.1.4. no-cache [x] 5.2.1.5. no-store [x] 5.2.1.6. no-transform [x] 5.2.1.7. only-if-cached [ ] 5.2.2. Response Cache-Control Directives [x] 5.2.2.1. max-age [x] 5.2.2.2. must-revalidate [x] 5.2.2.3. must-understand : http_cache only caches responses that have a known status code [x] 5.2.2.4. no-cache : only unqualified form is supported [x] 5.2.2.5. no-store [x] 5.2.2.6. no-transform [x] 5.2.2.7. private &quot; only unqualified form is supported [x] 5.2.2.8. proxy-revalidate [x] 5.2.2.9. public [x] 5.2.2.10. s-maxage [ ] 5.2.3. Cache Control Extensions [ ] 5.2.4. Cache Directive Registry [x] 5.3. Expires [x] 5.4. Pragma [ ] 5.5. Warning RFC5861: HTTP Cache-Control Extensions for Stale Content [x] 3. The stale-while-revalidate Cache-Control Extension [x] 4. The stale-if-error Cache-Control Extension (Only with the latest cowlib code, see comment above.) RFC9110: HTTP Semantics [ ] 13. Conditional Requests [ ] 13.1. Preconditions [ ] 13.1.1. If-Match [x] 13.1.2. If-None-Match [x] 13.1.3. If-Modified-Since [ ] 13.1.4. If-Unmodified-Since [x] 13.1.5. If-Range [x] 13.2. Evaluation of Preconditions [x] 13.2.1. When to Evaluate [x] 13.2.2. Precedence of Preconditions [ ] 14. Range Units [x] 14.1. Byte Ranges [x] 14.1.1. Range Specifiers [x] 14.1.2. Byte Ranges [x] 14.2. Range [x] 14.3. Accept-Ranges [ ] 14.4. Content-Range [ ] 14.5. Partial PUT [x] 14.6. Media Type multipart/byteranges","ref":"readme.html#conformance","title":"http_cache - Conformance","type":"extras"},{"doc":"Format with rebar3 format . Pay attention that some lines of the macros in src/http_cache.erl must be manually edited because of an issue in the format plugin.","ref":"readme.html#contributing","title":"http_cache - Contributing","type":"extras"},{"doc":"All notable changes to this project will be documented in this file. The format is based on Keep a Changelog and this project adheres to Semantic Versioning.","ref":"changelog.html","title":"CHANGELOG","type":"extras"},{"doc":"Added [ http_cache ] Added prevent_set_cookie option. Caching a response in a shared cache with set-cookie header now raises","ref":"changelog.html#0-3-1-2023-12-20","title":"CHANGELOG - [0.3.1] - 2023-12-20","type":"extras"},{"doc":"Changed [ http_cache ] Use external library http_cache_store_behaviour","ref":"changelog.html#0-3-0-2023-06-22","title":"CHANGELOG - [0.3.0] - 2023-06-22","type":"extras"},{"doc":"Added [ http_cache ] Export notifying_downloading/2 for future support of HTTP request collapsing Changed [ http_cache ] Options are now a map (was previously a proplist)","ref":"changelog.html#0-2-0-2023-04-25","title":"CHANGELOG - [0.2.0] - 2023-04-25","type":"extras"}]